// Functions for setting the clock and other user settings


int realIntensity(int intensity){
// Convert intensity value from 1-8 to the actual intensity which can be 0-255


  return map(intensity, 1, 8, MINITNENSITY, MAXINTENSITY);
  }

void doReset(){
  // This clears all settings by erasing the eeprom

  for (int i = 0 ; i < EEPROM.length() ; i++) {
    EEPROM.write(i, 255);
  }
}

void eepromSave(){
  // Save all of the current user settings
  
  byte sum;

  EEPROM.write(0, (byte)dayIntensity);
  EEPROM.write(1, (byte)nightIntensity);
  EEPROM.write(2, (byte)nightStart);
  EEPROM.write(3, (byte)dayStart);    
  sum = dayIntensity + nightIntensity + nightStart + dayStart;
  EEPROM.write(4, sum ^ 0xe5); // // write checksum after last settings byte
  
  }


bool eepromLoad(){
  // Load the user-saved settings from the eeprom
  // Returns false if the checksum was not valid, no data will be loaded

  byte sum;

  sum += (byte)EEPROM.read(0);
  sum += (byte)EEPROM.read(1); 
  sum += (byte)EEPROM.read(2); 
  sum += (byte)EEPROM.read(3); 
  
  if ( (sum^0xe5) == EEPROM.read(4)) {
      // Data is go, set the variables

      dayIntensity = (byte)EEPROM.read(0);
      nightIntensity = (byte)EEPROM.read(1); 
      nightStart = (byte)EEPROM.read(2); 
      dayStart = (byte)EEPROM.read(3); 
      return true;
    
  } 
 
  return false; // Checksum was invalid, settings not loaded
}


void doSetIntensityHours(){

  // Set the hour range for automatically setting the brightness based on time

  showIntensityHours();
  setModeTimeStamp = millis();
  int setDelay = 15000;

    while( millis() - setModeTimeStamp < setDelay ){

      
    bool changed = buttonMinute.update() | buttonHour.update();
      if (changed && buttonMinute.read() == LOW ){
        setDelay = 8000; // lower timeout once changes are made
        nightStart++;
  
        if(nightStart > 23){
          nightStart = 0;
          
          }
  

        showIntensityHours();
        setModeTimeStamp = millis();
      }
    
      if (changed && buttonHour.read() == LOW ){
        setDelay = 8000; // lower timeout once changes are made 
        dayStart++;
        
        if(dayStart > 23){

          dayStart = 0;
          
          }
        
        showIntensityHours();
        setModeTimeStamp = millis();
        
      }
    }

// New settings are applied, save the eeprom for loading on power up
  eepromSave();

}


void showIntensityHours(){
  // Show the hour when night starts and ends
  //   Start hour on the left, end our onthe right

  switch(nightStart){
    case 0:
      NIGHTHOUR12;
      NIGHTHOURAM;
      break;
    case 1:
      NIGHTHOUR1;
      NIGHTHOURAM;
      break;
    case 2:
      NIGHTHOUR2;
      NIGHTHOURAM;
      break;
    case 3:
      NIGHTHOUR3;
      NIGHTHOURAM;
      break;
    case 4:
      NIGHTHOUR4;
      NIGHTHOURAM;
      break;
    case 5:
      NIGHTHOUR5;
      NIGHTHOURAM;
      break;
    case 6:
      NIGHTHOUR6;
      NIGHTHOURAM;
      break;
    case 7:
      NIGHTHOUR7;
      NIGHTHOURAM;
      break;
    case 8:
      NIGHTHOUR8;
      NIGHTHOURAM;
      break;
    case 9:
      NIGHTHOUR9;
      NIGHTHOURAM;
      break;
    case 10:
      NIGHTHOUR10;
      NIGHTHOURAM;
      break;
    case 11:
      NIGHTHOUR11;
      NIGHTHOURAM;
      break;
    case 12:
      NIGHTHOUR12;
      break;
    case 13:
      NIGHTHOUR1;
      break;
    case 14:
      NIGHTHOUR2;
      break;
    case 15:
      NIGHTHOUR3;
      break;
    case 16:
      NIGHTHOUR4;
      break;
    case 17:
      NIGHTHOUR5;
      break;
    case 18:
      NIGHTHOUR6;
      break;
    case 19:
      NIGHTHOUR7;
      break;
    case 20:
      NIGHTHOUR8;
      break;
    case 21:
      NIGHTHOUR9;
      break;
    case 22:
      NIGHTHOUR10;
      break;
    case 23:
      NIGHTHOUR11;   
    
    }





  switch(dayStart){
    case 0:
      DAYHOUR12;
      DAYHOURAM;
      break;
    case 1:
      DAYHOUR1;
      DAYHOURAM;
      break;
    case 2:
      DAYHOUR2;
      DAYHOURAM;
      break;
    case 3:
      DAYHOUR3;
      DAYHOURAM;
      break;
    case 4:
      DAYHOUR4;
      DAYHOURAM;
      break;
    case 5:
      DAYHOUR5;
      DAYHOURAM;
      break;
    case 6:
      DAYHOUR6;
      DAYHOURAM;
      break;
    case 7:
      DAYHOUR7;
      DAYHOURAM;
      break;
    case 8:
      DAYHOUR8;
      DAYHOURAM;
      break;
    case 9:
      DAYHOUR9;
      DAYHOURAM;
      break;
    case 10:
      DAYHOUR10;
      DAYHOURAM;
      break;
    case 11:
      DAYHOUR11;
      DAYHOURAM;
      break;
    case 12:
      DAYHOUR12;
      break;
    case 13:
      DAYHOUR1;
      break;
    case 14:
      DAYHOUR2;
      break;
    case 15:
      DAYHOUR3;
      break;
    case 16:
      DAYHOUR4;
      break;
    case 17:
      DAYHOUR5;
      break;
    case 18:
      DAYHOUR6;
      break;
    case 19:
      DAYHOUR7;
      break;
    case 20:
      DAYHOUR8;
      break;
    case 21:
      DAYHOUR9;
      break;
    case 22:
      DAYHOUR10;
      break;
    case 23:
      DAYHOUR11;   
    
    }  
  
  // force brightness here ?????????
  applyMaskColor(0,255,0);
  }


void doSetIntensity(){
// Set the brightness for night and day


  showInt(nightIntensity, dayIntensity, 8); // show current levels, use maximum brighness in case they are too low
  setModeTimeStamp = millis();
  int setDelay = 15000;
  while( millis() - setModeTimeStamp < setDelay ){
      
    bool changed = buttonMinute.update() | buttonHour.update();
    if (changed && buttonMinute.read() == LOW ){
      setDelay = 8000; // lower timeout once setting happens
      nightIntensity++;
  
      if(nightIntensity > 8){
        nightIntensity = 1;
          
      }
  
      showInt(nightIntensity, dayIntensity, nightIntensity);   //realIntensity(nightIntensity)
      setModeTimeStamp = millis();
      }
    
      if (changed && buttonHour.read() == LOW ){
        setDelay = 8000; // lower timeout once setting happens
        dayIntensity++;
        
        if(dayIntensity > 8){

          dayIntensity = 1;
          
        }

        showInt(nightIntensity, dayIntensity,dayIntensity);         //realIntensity(dayIntensity)
        setModeTimeStamp = millis();
        
      }
    }

// Save the current settings
  eepromSave();

  }




void showInt(int nightIntensity, int dayIntensity, int brt){

// Show the current brightness/intensity, night on left, day on righ
//  Show it using the supplied 1-8 brightness value

  switch(nightIntensity){
    case 1:
      NIGHTINT1;
    break;
    case 2:
      NIGHTINT2;
    break;
    case 3:
      NIGHTINT3;
    break;
    case 4:
      NIGHTINT4;
    break;
    case 5:
      NIGHTINT5;
    break;
    case 6:
      NIGHTINT6;
    break;
    case 7:
      NIGHTINT7;
    break;
    case 8:
      NIGHTINT8;
    break;

    }
  switch(dayIntensity){
    case 1:
      DAYINT1;
    break;
    case 2:
      DAYINT2;
    break;
    case 3:
      DAYINT3;
    break;
    case 4:
      DAYINT4;
    break;
    case 5:
      DAYINT5;
    break;
    case 6:
      DAYINT6;
    break;
    case 7:
      DAYINT7;
    break;
    case 8:
      DAYINT8;
    break;

    }  


  matrix.setBrightness(realIntensity(brt));
  applyMaskColor(0,255,0);

  }
 



void doOption(int option){
// Execute the selected option
  
  switch (option) {
      case 0: //set 
        doSet();
        break;
      case 1: //Intensity
        doSetIntensity();
        break;
      case 2: //Intensity Hours
        doSetIntensityHours();
        break;
      case 3: // end, do nothing
        break;
      default: // shouldn't happen
        break;

  }  
  
  
  
  }

void showOption(int option){
// Show the selected option on the display
  
  switch (option) {
      case 0:
        SET;
        applyMaskColor(255,0,0);
        break;
      case 1:
        IGHT;
        applyMaskColor(0,255,0);
        break;
      case 2:
        NH;
        applyMaskColor(0,0,255);        
        break;
      case 3:
        END;
        applyMaskColor(255,255,255);        
        break;      
      default: // shouldn't happen
        break;

  }
}

void doSet(){

  // Set the clock
  
      setModeTimeStamp = millis();



    int setDelay = 15000; // start with 20 second delay, after making changes, lower timeout
    while(millis() - setModeTimeStamp < setDelay){

      thetime = RTC.now();
      showSetTime();

///////////////set mode///////////////////      


    
///////////////minute////////////////////    

      if (buttonMinute.update()){
    
     //   int value = buttonMinute.read();
    
        if(buttonMinute.read() == LOW){
          // increment minute here
          thetime = RTC.now();
          incrementSetMinute();
          setDelay = 8000; // lower timeout once setting has occured



          
          minuteButtonState = 1;
          buttonPressTimeStamp = millis();
          firstButtonPressTimeStamp = buttonPressTimeStamp;

          setModeTimeStamp = millis();


          
          } else {
             minuteButtonState = 0;
            
            }
        }


      if  ( minuteButtonState == 1 ) {
    
        int incDelay = 500;
    
    
        if (millis() - firstButtonPressTimeStamp > 3000 ){ // button held longer than 3 seconds, increase speed.
    
          incDelay = 100;
          }
    
        
        if ( millis() - buttonPressTimeStamp >= incDelay ) {

          // increment minute here
          thetime = RTC.now();
          incrementSetMinute();          
          setDelay = 8000; // lower timeout once setting has occured
          
          buttonPressTimeStamp = millis();
          setModeTimeStamp = millis();
        }
      }
//////////////////end minute////////////////
///////////////hour////////////////////    
     // changed = ;
      if (buttonHour.update()){
    

    
        if(buttonHour.read() == LOW){

          // increment hour here
          thetime = RTC.now();
          incrementSetHour();
          setDelay = 8000; // lower timeout once setting has occured

          
          hourButtonState = 1;
          buttonPressTimeStamp = millis();
          firstButtonPressTimeStamp = buttonPressTimeStamp;

          setModeTimeStamp = millis();


          
          } else {
             hourButtonState = 0;
            
            }
        }


      if  ( hourButtonState == 1 ) {
    
        int incDelay = 500;
    
    
        if (millis() - firstButtonPressTimeStamp > 3000 ){
    
          incDelay = 100;
          }
    
        
        if ( millis() - buttonPressTimeStamp >= incDelay ) {

          // increment hour here
          thetime = RTC.now();
          incrementSetHour(); 
          setDelay = 8000; // lower timeout once setting has occured                   

          buttonPressTimeStamp = millis();
          setModeTimeStamp = millis();
        }
      }
//////////////////end hour////////////////


      
     
///////////////end set mode///////////////////      
      }
  
  
  }

void incrementSetMinute(){

  // Increment the minute
  
  int newMinute = thetime.minute();
  newMinute++;
  int newHour = thetime.hour();
  

  if( newMinute > 59 ){
    newMinute = 0;
  }

  
  RTC.adjust(DateTime(2021, 1, 1, newHour, newMinute, 0));
  timeValid = true; //time has been set
  }


void incrementSetHour(){

  // Increment the hour
  
  int newMinute = thetime.minute();

  int newHour = thetime.hour();
  newHour++;
  if(newHour > 23){
    newHour = 0;
  }
 

   
  int newSecond = thetime.second(); // don't touch the minute when setting hours
  
  RTC.adjust(DateTime(2021, 1, 1, newHour, newMinute, newSecond));
  timeValid = true; //time has been set
  }



void showSetTime(){
  
// Display the time
// Hours as number of lit letters on first 2 rows in 3 groups of 4
// AM indicated by A on third row
// Seconds as 7-segment display on bottom part of display

bottomMask = 0x0;
topMask = 0x0;


    int displayHour = thetime.hour(); // for displaying the hour, convert to 12 hour

    if( displayHour >= 12 ){ // still 24 hour here Set AM/PM
      SETHOURPM;
      } else {
        SETHOURAM;
      }
      

    // Use 12 hour display
    if(displayHour > 12){ 
      displayHour = displayHour - 12;
    }

    if(displayHour == 0){
      displayHour = 12;
      }


 // Show the time
  switch (displayHour) {
      case 1:
        SETHOUR1;
        break;
      case 2:
        SETHOUR2;
        break;
      case 3:
        SETHOUR3;
        break;

      case 4:
        SETHOUR4;
        break;      
      case 5:
        SETHOUR5;
        break;
      case 6:
        SETHOUR6;
        break;
      case 7:
        SETHOUR7;
        break;
      case 8:
        SETHOUR8;
        break;
 
      case 9:
        SETHOUR9;
        break;      
      case 10:
        SETHOUR10;
        break;
   
      case 11:
        SETHOUR11;
        break;
     case 12:
        SETHOUR12;
        break;

   }


  
  switch (thetime.minute()/10) {
      case 0:
        SETMIN00;
        break;
      case 1:
        SETMIN10;
        break;
      case 2:
        SETMIN20;
        break;
      case 3:
        SETMIN30;
        break;
      case 4:
        SETMIN40;
        break;
      case 5:
        SETMIN50;
        break;
   }




  switch (thetime.minute()%10) {
      case 0:
        SETMIN0;
        break;
      case 1:
        SETMIN1;
        break;
      case 2:
        SETMIN2;
        break;
      case 3:
        SETMIN3;
        break;

      case 4:
        SETMIN4;
        break;      
      case 5:
        SETMIN5;
        break;
      case 6:
        SETMIN6;
        break;
      case 7:
        SETMIN7;
        break;
      case 8:
        SETMIN8;
        break;
      case 9:
        SETMIN9;
        break;
   }


// Show the time, special function colors the hours/AM/seconds
  applyMaskColorForSet();
  
}
